/*
Module : DDXFile.cpp
Purpose: Implementation for a CodifyButton" to the better named "CDDXFileModifyButton".
                          3. Enabled state of control is now reflected to buddy button edit control. Thanks to 
                          "matro" for pointing out this problem.
                          4. Made the "CGetFilenameControl::Edit" virtual for easier customisation.
                          5. Made all strings used by the classes easier to modify by making them virtual
                          methods of the class
                          6. Changed a class name from "CGetFileNameDialog" to the better named "CDDXFileFileNameDialog"
                          7. Made the "CGetFilenameControl::AddEditButton" method virtual for easier customisation.
         PJN / 19-07-2006 1. Updated copyright details
                          2. Optimized CDDXFileModifyButton constructor code
                          3. Updated documentation to use the same style as the web site.
                          4. Optimized CDDXFileFileNameDialog constructor code
                          5. Addition of a CDDXFILE_EXT_CLASS preprocessor macro to allow the classes to be easily 
                          added to an extension dll.
                          6. Removed now defunct Win32s support code
                          7. Removed support for showing the old style Win3.1 file open dialog
                          8. Changed the names of the constants used for DDX_GetFilenameControl
                          9. Replaced all calls to CFile::GetStatus with GetFileAttributes
                          10. Verified code clean compiles in VC2005
         PJN / 29-07-2006 1. Initial enabled state of the edit control is now reflected into the edit button
                          when AddEditButton is called. 
         PJN / 11-08-2006 1. Reworked all the use of string resources. Now all the string resources used
                          by the control are all loaded in one single virtual function. In addition if you 
                          do not want to use inheritance, all the strings can be customized via Get/Set 
                          methods.
                          2. Broke the DDV functionality into separate discreet DDV functions
                          3. The dialog control id of the edit button can now be specified via the SubclassEdit
                          and DDX_FilenameControl functions
         PJN / 20-08-2006 1. Control now takes advantage of the AutoCompletion functionality (IAutoComplete et al) 
                          built into Windows to improve the UI experience. If enabled then a drop down list
                          will appear underneath the edit control which allows the user to quickly select a specific
                          file. The code uses the COM interfaces (IAutoComplete) to AutoComplete instead of the 
                          more simplified functionality provided by SHAutoComplete. This allows the code to customize
                          the auto completion UI so that only file system files are selectable.
         PJN / 13-11-2007 1. Minor update to display HRESULT's correctly.
                          2. Minor tidy up to remove old style AppWizard comments
         PJN / 21-12-2008 1. Updated copyright details
                          2. Removed VC 6 style AppWizard comments from the code
                          3. InitializeAutoComplete now returns a HRESULT instead of a BOOL
                          4. Updated DDX_FileControl code to be consistent with DDX_Control in VC 2008 
                          5. The code has now been updated to support VC 2005 or later only
                          6. Code now compiles cleanly using Code Analysis (/analyze)
                          7. The code now can use the Vista style of file dialog provided by VC 2008
                          8. Fixed a bug in the sample app where the various checkbox UI states
                          were not initially in sync with the control.
         PJN / 10-09-2011 1. Updated copyright details.
                          2. Verified code compiles cleanly on VC 2010
                          3. Fixed an access violation bug in CDDXFileModifyButton::PreTranslateMessage if the 
                          IDS_DDXFILE_EDIT_TEXT string resource is not loaded
         PJN / 01-03-2014 1. Updated copyright details.
                          2. Updated MFC message maps in sample app and in DDXFile module to use C++ standard 
                          for pointers to member methods.
                          3. Reworked DDX_FilenameControl to use DDX_Control for most of its functionality
                          4. Reworked how the code exposes the bVistaStyle support in the MFC CFileDialog class. 
                          The code now uses the MSC C++ __if_exists / __if_not_exists statements
                          5. Updated the code to clean compile on VC 2013 & 2012.
                          6. Updated the code to clean compile using /analyze
         PJN / 23-12-2014 1. If the main edit control is now hidden, the buddy button is now also hidden. The 
                          corollary of if the main edit control is now shown, then the buddy button is now also
                          shown. Thanks to Rene Laviolette for requesting this update.
         PJN / 11-11-2015 1. Verified the code compiles cleanly on VC 2015.
                          2. Updated the sample apps main icon.
                          3. Updated copyright details.
         PJN / 24-09-2017 1. Updated copyright details.
                          2. Replaced NULL throughout the codebase with nullptr. This means that the minimum
                          requirement for the framework is now VC 2010.
                          3. Reworked the code in CGetFilenameControl::InitializeAutoComplete to use
                          ATL::CComPtr::CoCreateInstance.
                          4. All the code has had SAL annotations added
         PJN / 09-07-2018 1. Updated copyright details.
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean that
                          the code will now only compile on VC 2017 or later.
         PJN / 19-05-2019 1. Updated copyright details.
                          2. Updated the code to clean compile on VC 2019
         PJN / 21-12-2019 1. Fixed various Clang-Tidy static code analysis warnings in the code.
         PJN / 14-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 03-05-2020 1. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 01-04-2022 1. Updated copyright details.
                          2. Updated the code to use C++ uniform initialization for all variable declarations

Copyright (c) 1997 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "DDXFile.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CDDXFileModifyButton, CButton) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
  ON_CONTROL_REFLECT(BN_CLICKED, &CDDXFileModifyButton::OnClicked)
END_MESSAGE_MAP()

CDDXFileModifyButton::CDDXFileModifyButton() noexcept : m_pBuddy{nullptr},
                                                        m_bFirstPreTranslateMessageCall{true}
{
}

void CDDXFileModifyButton::SetBuddy(_In_opt_ CGetFilenameControl* pBuddy) noexcept
{
  m_pBuddy = pBuddy;
}

BOOL CDDXFileModifyButton::PreTranslateMessage(MSG* pMsg)
{
  //Validate our parameters
#pragma warning(suppress: 26496)
  AFXASSUME(m_pBuddy != nullptr);

  //create the tooltip
  if (m_bFirstPreTranslateMessageCall)
  {
    CString sTTText{m_pBuddy->GetModifyButtonToolTipText()};
    if (sTTText.GetLength())
    {
      m_ToolTip.Create(this);
      m_ToolTip.Activate(TRUE);
      m_ToolTip.AddTool(this, sTTText);
    }
    m_bFirstPreTranslateMessageCall = false;
  }

  //give the tooltip a chance to handle the message if is is valid
  if (m_ToolTip.GetSafeHwnd() != nullptr)
    m_ToolTip.RelayEvent(pMsg);

  return __super::PreTranslateMessage(pMsg);
}

void CDDXFileModifyButton::OnClicked()
{
  //Validate our parameters
#pragma warning(suppress: 26496)
  AFXASSUME(m_pBuddy != nullptr);

  //Ask the buddy to show the dialog
  m_pBuddy->Edit();
}


CDDXFileFileNameDialog::CDDXFileFileNameDialog(_In_ BOOL bOpenFileDialog, _In_opt_ LPCTSTR lpszDefExt, _In_opt_ LPCTSTR lpszFileName, _In_ DWORD dwFlags, _In_opt_ LPCTSTR lpszFilter, _In_opt_ CWnd* pParentWnd, _In_ DWORD dwSize, _In_ BOOL bVistaStyle) : CFileDialog{bOpenFileDialog, lpszDefExt, lpszFileName, dwFlags, lpszFilter, pParentWnd, dwSize, bVistaStyle},
                                                                                                                                                                                                                                                              m_pBuddy{nullptr}
{
}

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CDDXFileFileNameDialog, CFileDialog) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
END_MESSAGE_MAP()

void CDDXFileFileNameDialog::SetBuddy(_In_opt_ CGetFilenameControl* pBuddy) noexcept
{
  m_pBuddy = pBuddy;
}

void CDDXFileFileNameDialog::OnInitDone()
{
  //Validate our parameters
#pragma warning(suppress: 26496)
  AFXASSUME(m_pBuddy != nullptr);

  //Modify the text on the IDOK button to be the specified value, nominally "Ok"
  CString sOkCaption{m_pBuddy->GetCommonDialogOkCaption()};
  if (sOkCaption.GetLength())
    SetControlText(IDOK, sOkCaption);
}

BOOL CDDXFileFileNameDialog::OnFileNameOK()
{
  //Let the base class do its thing
  BOOL bVal{__super::OnFileNameOK()};

  if (!bVal) //if the filename passed the parent tests then put it through our own tests
  {
    //check for overwrite if that flag is used, since the file open
    //common dialog will not bother looking at this flag
    if (m_ofn.Flags & OFN_OVERWRITEPROMPT)
    {
      //Retrieve the filename selected
      CString sPath{GetFolderPath()};

      //check to see if the file already exists
      if (GetFileAttributes(sPath) != INVALID_FILE_ATTRIBUTES)
      {
        ASSERT(m_pBuddy != nullptr);
        CString sMessage{m_pBuddy->GetOverwritePromptString(sPath)};

        //ensure the title is the same as the common dialog caption
        CString sCaption{m_pBuddy->GetCommonDialogTitle()};
        bVal = (MessageBox(sMessage, sCaption, MB_YESNO | MB_ICONQUESTION) != IDYES);
      }
    }
  }

  return bVal;
}


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CGetFilenameControl, CEdit) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_ENABLE()
  ON_WM_SHOWWINDOW()
END_MESSAGE_MAP()

CGetFilenameControl::CGetFilenameControl() noexcept : m_dwCommonDialogFlags{OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT},
                                                      m_dwCommonDialogSize{0},
                                                      m_bVistaStyle{true},
                                                      m_bAutoComplete{true},
                                                      m_dwAutoCompleteFlags{ACO_AUTOSUGGEST | ACO_FILTERPREFIXES}
{
}

bool CGetFilenameControl::SubclassEdit(_In_ HWND hEdit, _In_ UINT nModifyButtonID)
{
  //test our inputs
  if (!IsWindow(hEdit))
  {
    ASSERT(false);
    TRACE(_T("CGetFilenameControl::SubclassEdit, window handle is invalid!\n"));
    return false;
  }

  //subclass the control
  if (SubclassWindow(hEdit))
  {
    LoadStringResources();
    InitializeAutoComplete(); //Failing to setup auto complete is not considered a critical error
    return AddModifyButton(nModifyButtonID);
  }
  else
  {
    TRACE(_T("CGetFilenameControl::SubclassEdit, Could not subclass edit control!\n"));
    ASSERT(false);
    return false;
  }
}

void CGetFilenameControl::SetAutoComplete(_In_ bool bAutoComplete)
{
  //Save the setting in the member variable
  m_bAutoComplete = bAutoComplete;

  //Also let the auto complete object know
  if (m_pAutoCompleteObject != nullptr)
  {
    if (m_bAutoComplete)
    {
      const HRESULT hr{m_pAutoCompleteObject->SetOptions(m_dwAutoCompleteFlags)};
      if (FAILED(hr))
        TRACE(_T("CGetFilenameControl::SetAutoComplete, failed to set AutoComplete object options, Error:%08X\n"), hr);
    }
    else
    {
      const HRESULT hr{m_pAutoCompleteObject->SetOptions(ACO_NONE)};
      if (FAILED(hr))
        TRACE(_T("CGetFilenameControl::SetAutoComplete, failed to set AutoComplete object options, Error:%08X\n"), hr);
    }
  }
}

HRESULT CGetFilenameControl::InitializeAutoComplete()
{
  if (m_bAutoComplete)
  {
    //Create the auto complete object
    HRESULT hr{m_pAutoCompleteObject.CoCreateInstance(CLSID_AutoComplete, nullptr, CLSCTX_INPROC_SERVER)};
    if (FAILED(hr))
    {
      TRACE(_T("CGetFilenameControl::InitializeAutoComplete, failed to create AutoComplete object, Error:%08X\n"), hr);
      return hr;
    }

    //Create the auto complete source object
    hr = m_pAutoCompleteSourceObject.CoCreateInstance(CLSID_ACListISF, nullptr, CLSCTX_INPROC_SERVER);
    if (FAILED(hr))
    {
      TRACE(_T("CGetFilenameControl::InitializeAutoComplete, failed to create AutoComplete filesystem source object, Error:%08X\n"), hr);
      return hr;
    }

    //Tell the auto complete source object to only return file system entries
    hr = m_pAutoCompleteSourceObject->SetOptions(ACLO_FILESYSONLY);
    if (FAILED(hr))
    {
      TRACE(_T("CGetFilenameControl::InitializeAutoComplete, failed to set AutoComplete filesystem source object options, Error:%08X\n"), hr);
      return hr;
    }

    //Finally set the options and hook up the edit control
    hr = m_pAutoCompleteObject->SetOptions(m_dwAutoCompleteFlags);
    if (FAILED(hr))
    {
      TRACE(_T("CGetFilenameControl::InitializeAutoComplete, failed to set AutoComplete object options, Error:%08X\n"), hr);
      return hr;
    }
    hr = m_pAutoCompleteObject->Init(GetSafeHwnd(), m_pAutoCompleteSourceObject, nullptr, nullptr);
    if (FAILED(hr))
    {
      TRACE(_T("CGetFilenameControl::InitializeAutoComplete, failed to initialize AutoComplete object, Error:%08X\n"), hr);
      return hr;
    }
  }

  return S_OK;
}

bool CGetFilenameControl::AddModifyButton(_In_ UINT nModifyButtonID)
{
  //Validate our parameters
  CWnd* pParent{GetParent()};
#pragma warning(suppress: 26496)
  AFXASSUME(pParent != nullptr);

  //Work out the dimensions of the modify button and the resized edit control
  CRect editRect;
  GetWindowRect(editRect);
  pParent->ScreenToClient(editRect);
  CRect btnRect;
  btnRect.top = editRect.top;
  btnRect.bottom = editRect.bottom;
  btnRect.right  = editRect.right;

  //Calculate the width of the button based on the text we are going to put on in
  CDC* pDC{pParent->GetDC()};
  int nButtonWidth{0};
  if (pDC != nullptr)
  {
    //Setup the DC
    CFont* pNewFont{pParent->GetFont()};
    CFont* pOldFont{pDC->SelectObject(pNewFont)};

    //Get the button width
    CSize SizeText{pDC->GetTextExtent(_T("    "), 4)}; //We allow for some space around the button caption
    pDC->LPtoDP(&SizeText);
    nButtonWidth = SizeText.cx;
    SizeText = pDC->GetTextExtent(m_sModifyButtonCaption, m_sModifyButtonCaption.GetLength());
    pDC->LPtoDP(&SizeText);
    nButtonWidth += SizeText.cx;

    //Restore the DC
    pDC->SelectObject(pOldFont);
  }
  else
    nButtonWidth = btnRect.Height()*8/10; //width is 8/10 of height

  //Now that we have the width we can fill out the rest of the buttons rect position
  btnRect.left = btnRect.right - nButtonWidth;

  //Tidy up the device context
  if (pDC != nullptr)
#pragma warning(suppress: 26486)
    pParent->ReleaseDC(pDC);

  //Resize the edit control
  SetWindowPos(nullptr, 0, 0, editRect.Width() - btnRect.Width() - 1, editRect.Height(), SWP_NOACTIVATE | SWP_NOMOVE | SWP_SHOWWINDOW | SWP_NOZORDER);

  //dynamically create the modify button control
  DWORD dwButtonStyles{WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_GROUP};
  if (GetStyle() & WS_DISABLED)
    dwButtonStyles |= WS_DISABLED;
#pragma warning(suppress: 26486)
  if (!m_ctrlModifyButton.Create(m_sModifyButtonCaption, dwButtonStyles, btnRect, pParent, nModifyButtonID))
    return false;

  //Set the tab order of the button
  m_ctrlModifyButton.SetWindowPos(this, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);

  //tell the button to call this class when it is clicked
  m_ctrlModifyButton.SetBuddy(this);

  //ensure it is using the same font as the parent
  m_ctrlModifyButton.SetFont(pParent->GetFont());

  return true;
}

#pragma warning(suppress: 26434)
void CGetFilenameControl::OnEnable(BOOL bEnable)
{
  //Let the base class do its thing
  __super::OnEnable(bEnable);

  //reflect the enabled state of this control to the buddy button control
  m_ctrlModifyButton.EnableWindow(bEnable);
}

#pragma warning(suppress: 26434)
void CGetFilenameControl::OnShowWindow(BOOL bShow, UINT nStatus)
{
  //Let the base class do its thing
  __super::OnShowWindow(bShow, nStatus);

  //reflect the visibility state of this control to the buddy button control
  m_ctrlModifyButton.ShowWindow(bShow ? SW_SHOW : SW_HIDE);
}

void CGetFilenameControl::Edit()
{
  //Retrieve the current filename to use as the initial value
  CString sCurrentFilename;
  GetWindowText(sCurrentFilename);

  //Make sure the browse button at least works if a directory is
  //currently selected
  const int nCurrentFilenameLength{sCurrentFilename.GetLength()};
  if (nCurrentFilenameLength)
  {
    if (sCurrentFilename[nCurrentFilenameLength - 1] == _T('\\'))
      sCurrentFilename += _T("*.*");
  }

  //Bring up the common dialog
  CDDXFileFileNameDialog dlg{TRUE, nullptr, sCurrentFilename, m_dwCommonDialogFlags, m_sCommonDialogExtFilter, GetParent(), m_dwCommonDialogSize, m_bVistaStyle};
  dlg.SetBuddy(this);

  //Modify the title to the desired value
  if (m_sCommonDialogTitle.GetLength())
    dlg.m_ofn.lpstrTitle = m_sCommonDialogTitle;

  //Modify the text on the OK button to be the specified value, since it will not be done via OnInitDone
  //which is where we change OK button text for non Vista style File dialogs. This is because OnInitDone
  //is not supported for Vista style File dialogs
  if (dlg.UsingVistaStyle())
  {
    CComPtr<IFileOpenDialog> pFileOpenDialog;
    pFileOpenDialog.Attach(dlg.GetIFileOpenDialog());
    if (pFileOpenDialog != nullptr)
      pFileOpenDialog->SetOkButtonLabel(CStringW(m_sCommonDialogOkCaption));
  }

  //bring up the dialog and if the user hits ok, then set the text in this control to the new filename
  if (dlg.DoModal() == IDOK)
    SetWindowText(dlg.GetPathName());
}

bool CGetFilenameControl::LoadStringResources()
{
  //Use the default strings, a derived class can change this implementation or call the "Set.." functions directly
  if (!m_sCommonDialogOkCaption.LoadString(IDS_DDXFILE_OK))
    return false;
  if (!m_sModifyButtonCaption.LoadString(IDS_DDXFILE_EDIT_TEXT))
    return false;
  if (!m_sModifyButtonToolTipText.LoadString(IDS_DDXFILE_TT_MODIFY))
    return false;
  if (!m_sCommonDialogTitle.LoadString(IDS_DDXFILE_PLEASE_SPECIFY_A_FILENAME))
    return false;
  if (!m_sCommonDialogExtFilter.LoadString(IDS_DDXFILE_ALL_EXTENSION_FILTER))
    return false;
  return true;
}

CString CGetFilenameControl::GetOverwritePromptString(_In_ const CString& sFilename)
{
  CString sReturn;
  AfxFormatString1(sReturn, IDP_DDXFILE_OVERWRITE_PROMPT, sFilename);
  return sReturn;
}

bool DDX_FilenameControl(_In_ CDataExchange* pDX, _In_ int nIDC, _Inout_ CGetFilenameControl& rControl, _In_ UINT nModifyButtonID)
{
  //Check to see if the control already exists at the start of this function
  const bool bControlAlreadyExists{rControl.GetSafeHwnd() != nullptr};

  //Let DDX_Control do most of the work
  DDX_Control(pDX, nIDC, rControl);

  //What will be the return value from this method (assume the worst)
  bool bSuccess{false};

  //Setup the other various settings if required
  if (rControl.GetSafeHwnd() != nullptr)
  {
    if (!bControlAlreadyExists)
    {
      rControl.LoadStringResources();
      rControl.InitializeAutoComplete();
      bSuccess = rControl.AddModifyButton(nModifyButtonID);
    }
    else
      bSuccess = true;
  }

  return bSuccess;
}

#pragma warning(suppress: 26461)
void DDX_FilenameValue(_In_ CDataExchange* pDX, _In_ CGetFilenameControl& rControl, _Inout_ CString& sFile)
{
  if (pDX->m_bSaveAndValidate)
    rControl.GetWindowText(sFile);
  else
    rControl.SetWindowText(sFile);
}

#pragma warning(suppress: 26429)
void DDV_FilenameControlNotFolder(_In_ CDataExchange* pDX, _In_ const CGetFilenameControl& rControl, _In_ UINT nFailureResourceID, _In_ UINT nHelpID)
{
  if (pDX->m_bSaveAndValidate)
  {
    CString sFile;
    rControl.GetWindowText(sFile);
    const DWORD dwAttributes{GetFileAttributes(sFile)};
    if ((dwAttributes != INVALID_FILE_ATTRIBUTES) && (dwAttributes & FILE_ATTRIBUTE_DIRECTORY))
    {
      AfxMessageBox(nFailureResourceID, MB_OK | MB_ICONEXCLAMATION, nHelpID);
#pragma warning(suppress: 26486)
      pDX->PrepareEditCtrl(rControl.GetDlgCtrlID());
      pDX->Fail();
    }
  }
}

#pragma warning(suppress: 26429)
void DDV_FilenameControlMustExist(_In_ CDataExchange* pDX, _In_ const CGetFilenameControl& rControl, _In_ UINT nFailureResourceID, _In_ UINT nHelpID)
{
  if (pDX->m_bSaveAndValidate)
  {
    CString sFile;
    rControl.GetWindowText(sFile);
    const DWORD dwAttributes{GetFileAttributes(sFile)};
    if (dwAttributes == INVALID_FILE_ATTRIBUTES)
    {
      AfxMessageBox(nFailureResourceID, MB_OK | MB_ICONEXCLAMATION, nHelpID);
#pragma warning(suppress: 26486)
      pDX->PrepareEditCtrl(rControl.GetDlgCtrlID());
      pDX->Fail();
    }
  }
}

#pragma warning(suppress: 26429)
void DDV_FilenameControlOverwritePrompt(_In_ CDataExchange* pDX, _In_ CGetFilenameControl& rControl, _In_ UINT nHelpID)
{
  if (pDX->m_bSaveAndValidate)
  {
    CString sFile;
    rControl.GetWindowText(sFile);
    const DWORD dwAttributes{GetFileAttributes(sFile)};
    if (dwAttributes != INVALID_FILE_ATTRIBUTES)
    {
      //Prompt the user if we can overwrite and if we get a negative indication, then fail the validation
      if (AfxMessageBox(rControl.GetOverwritePromptString(sFile), MB_YESNO | MB_ICONQUESTION, nHelpID) != IDYES)
      {
#pragma warning(suppress: 26486)
        pDX->PrepareEditCtrl(rControl.GetDlgCtrlID());
        pDX->Fail();
      }
    }
  }
}

#pragma warning(suppress: 26429)
void DDV_FilenameControlNotEmpty(_In_ CDataExchange* pDX, _In_ const CGetFilenameControl& rControl, _In_ UINT nFailureResourceID, _In_ UINT nHelpID)
{
  if (pDX->m_bSaveAndValidate)
  {
    CString sFile;
    rControl.GetWindowText(sFile);

    //Check to see if the file exists
    if (sFile.IsEmpty())
    {
      AfxMessageBox(nFailureResourceID, MB_OK | MB_ICONEXCLAMATION, nHelpID);
#pragma warning(suppress: 26486)
      pDX->PrepareEditCtrl(rControl.GetDlgCtrlID());
      pDX->Fail();
    }
  }
}
