/*
Module : DDXFile.h
Purpose: Defines the interface for a MFC control to get a filename using the file open/save as common dialogs

Copyright (c) 1997 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Defines //////////////////////////////////////////////////

#pragma once

#ifndef _DDXFILE_H__
#define _DDXFILE_H__

#ifndef CDDXFILE_EXT_CLASS
#define CDDXFILE_EXT_CLASS
#endif //#ifndef CDDXFILE_EXT_CLASS

#ifndef CDDXFILE_EXT_API
#define CDDXFILE_EXT_API
#endif //#ifndef CDDXFILE_EXT_API

#ifndef __ATLWIN_H__
#pragma message("DDXFile as of v1.21 requires ATL support to implement its functionality. If your project is MFC only, then you need to update it to include ATL support")
#endif //#ifndef __ATLWIN_H__

#if WINVER < 0x0600
#pragma message("DDXFile should be compiled with WINVER >= 0x0600 to allow it to support full Vista style file dialog functionality")
#endif


//////////////////// Includes /////////////////////////////////////////////////

#ifndef _SHLOBJ_H_
#pragma message("To avoid this message, please put shlobj.h in your pre compiled header (normally stdafx.h)")
#include <shlobj.h>
#endif //#ifndef _SHLOBJ_H_


//////////////////// Classes //////////////////////////////////////////////////

//forward declaration
class CGetFilenameControl;

class CDDXFILE_EXT_CLASS CDDXFileModifyButton : public CButton
{
public:
//Constructors / Destructors
  CDDXFileModifyButton() noexcept;
  CDDXFileModifyButton(const CDDXFileModifyButton&) = delete;
  CDDXFileModifyButton(CDDXFileModifyButton&&) = delete;
  ~CDDXFileModifyButton() = default; //NOLINT(modernize-use-override)

//Methods
  CDDXFileModifyButton& operator=(const CDDXFileModifyButton&) = delete;
  CDDXFileModifyButton& operator=(CDDXFileModifyButton&&) = delete;
  void SetBuddy(_In_opt_ CGetFilenameControl* pBuddy) noexcept;

protected:
//Methods
  BOOL PreTranslateMessage(MSG* pMsg) override;

//Member variables
  CGetFilenameControl* m_pBuddy;
  bool m_bFirstPreTranslateMessageCall;
  CToolTipCtrl m_ToolTip;

//Message handlers
  afx_msg void OnClicked();

  DECLARE_MESSAGE_MAP()
};


class CDDXFILE_EXT_CLASS CDDXFileFileNameDialog : public CFileDialog
{
public:
//Constructors / Destructors
  CDDXFileFileNameDialog(_In_ BOOL bOpenFileDialog, _In_opt_ LPCTSTR lpszDefExt = nullptr, _In_opt_ LPCTSTR lpszFileName = nullptr,
                         _In_ DWORD dwFlags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, _In_opt_ LPCTSTR lpszFilter = nullptr,
                         _In_opt_ CWnd* pParentWnd = nullptr, _In_ DWORD dwSize = 0, _In_ BOOL bVistaStyle = TRUE);
  CDDXFileFileNameDialog(const CDDXFileFileNameDialog&) = delete;
  CDDXFileFileNameDialog(CDDXFileFileNameDialog&&) = delete;
  ~CDDXFileFileNameDialog() = default; //NOLINT(modernize-use-override)

//Methods
  CDDXFileFileNameDialog& operator=(const CDDXFileFileNameDialog&) = delete;
  CDDXFileFileNameDialog& operator=(CDDXFileFileNameDialog&&) = delete;
  void SetBuddy(_In_opt_ CGetFilenameControl* pBuddy) noexcept;
  _NODISCARD BOOL UsingVistaStyle() const noexcept { return m_bVistaStyle; };

protected:
//Methods
  void OnInitDone() override;
  BOOL OnFileNameOK() override;

//Member variables
  CGetFilenameControl* m_pBuddy;

  DECLARE_MESSAGE_MAP()
};


class CDDXFILE_EXT_CLASS CGetFilenameControl : public CEdit
{
public:
//Constructors / Destructors
  CGetFilenameControl() noexcept;
  CGetFilenameControl(const CGetFilenameControl&) = delete;
  CGetFilenameControl(CGetFilenameControl&&) = delete;
  ~CGetFilenameControl() = default; //NOLINT(modernize-use-override)

//Methods
  CGetFilenameControl& operator=(const CGetFilenameControl&) = delete;
  CGetFilenameControl& operator=(CGetFilenameControl&&) = delete;
  bool SubclassEdit(_In_ HWND hEdit, _In_ UINT nModifyButtonID = 0xFFFFFFFF);
  CDDXFileModifyButton& GetModifyButton() noexcept { return m_ctrlModifyButton; };

//Accessors/Mutators
  //Set/Get the caption used in the common dialog
  void SetCommonDialogTitle(_In_ const CString& sTitle) { m_sCommonDialogTitle = sTitle; };
  _NODISCARD CString GetCommonDialogTitle() const { return m_sCommonDialogTitle; };

  //Set/Get the extension filter string used in the common dialog
  void SetCommonDialogExtensionFilter(_In_ const CString& sFilter) { m_sCommonDialogExtFilter = sFilter; };
  _NODISCARD CString GetCommonDialogExtensionFilter() const { return m_sCommonDialogExtFilter; };

  //Get/Set the caption used for the "Ok" button in the common dialog
  void SetCommonDialogOkCaption(_In_ const CString& sCaption) { m_sCommonDialogOkCaption = sCaption; };
  _NODISCARD CString GetCommonDialogOkCaption() const { return m_sCommonDialogOkCaption; };

  //Get/Set the size used for the common dialog structure (Defaults to 0 which means use the OS default value)
  void SetCommonDialogSize(_In_ DWORD dwSize) noexcept { m_dwCommonDialogSize = dwSize; };
  _NODISCARD DWORD GetCommonDialogSize() const noexcept { return m_dwCommonDialogSize; };

  //Get/Set whether Vista style common dialogs are used
  void SetCommonVistaStyle(_In_ bool bVistaStyle) noexcept { m_bVistaStyle = bVistaStyle; };
  _NODISCARD bool GetCommonVistaStyle() const noexcept { return m_bVistaStyle; };

  //Get/Set the Modify button caption 
  void SetModifyButtonCaption(_In_ const CString& sCaption) { m_sModifyButtonCaption = sCaption; };
  _NODISCARD CString GetModifyButtonCaption() const { return m_sModifyButtonCaption; };

  //Get/Set the ToolTip text for the edit button
  void SetModifyButtonToolTipText(_In_ const CString& sText) { m_sModifyButtonToolTipText = sText; };
  _NODISCARD CString GetModifyButtonToolTipText() const { return m_sModifyButtonToolTipText; };

  //Get/Set the common dialog flags value
  void SetCommonDialogFlags(_In_ DWORD dwFlags) noexcept { m_dwCommonDialogFlags = dwFlags; };
  _NODISCARD DWORD GetCommonDialogFlags() const noexcept { return m_dwCommonDialogFlags; };

  //Auto completion settings
  void SetAutoComplete(_In_ bool bAutoComplete);
  _NODISCARD bool GetAutoComplete() const noexcept { return m_bAutoComplete; };
  void SetAutoCompleteFlags(_In_ DWORD dwFlags) noexcept { m_dwAutoCompleteFlags = dwFlags; };
  _NODISCARD DWORD GetAutoCompleteFlags() const noexcept { return m_dwAutoCompleteFlags; };

//Misc virtual functions
  virtual CString GetOverwritePromptString(_In_ const CString& sFilename);
  virtual void Edit(); //Bring up the file picker dialog
  virtual bool AddModifyButton(_In_ UINT nModifyButtonID);
  virtual bool LoadStringResources();
  virtual HRESULT InitializeAutoComplete();

protected:
//Member variables
  CDDXFileModifyButton m_ctrlModifyButton;
  CString m_sCommonDialogTitle;
  CString m_sCommonDialogExtFilter;
  CString m_sCommonDialogOkCaption;
  CString m_sModifyButtonCaption;
  CString m_sModifyButtonToolTipText;
  DWORD m_dwCommonDialogFlags;
  DWORD m_dwCommonDialogSize;
  bool m_bVistaStyle;
  bool m_bAutoComplete;
  DWORD m_dwAutoCompleteFlags;
  ATL::CComPtr<IAutoComplete2> m_pAutoCompleteObject;
  ATL::CComPtr<IACList2> m_pAutoCompleteSourceObject;

//Message handlers
  afx_msg void OnEnable(BOOL bEnable);
  afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);

  DECLARE_MESSAGE_MAP()
};


/////////////// MFC Data exchange routines //////////////////////////

bool CDDXFILE_EXT_API DDX_FilenameControl(_In_ CDataExchange* pDX, _In_ int nIDC, _Inout_ CGetFilenameControl& rControl, _In_ UINT nModifyButtonID = 0xFFFFFFFF);
void CDDXFILE_EXT_API DDX_FilenameValue(_In_ CDataExchange* pDX, _In_ CGetFilenameControl& rControl, _Inout_ CString& sFile);
void CDDXFILE_EXT_API DDV_FilenameControlNotFolder(_In_ CDataExchange* pDX, _In_ const CGetFilenameControl& rControl, _In_ UINT nFailureResourceID, _In_ UINT nHelpID = 0xFFFFFFFF);
void CDDXFILE_EXT_API DDV_FilenameControlMustExist(_In_ CDataExchange* pDX, _In_ const CGetFilenameControl& rControl, _In_ UINT nFailureResourceID, _In_ UINT nHelpID = 0xFFFFFFFF);
void CDDXFILE_EXT_API DDV_FilenameControlOverwritePrompt(_In_ CDataExchange* pDX, _In_ CGetFilenameControl& rControl, _In_ UINT nHelpID = 0xFFFFFFFF);
void CDDXFILE_EXT_API DDV_FilenameControlNotEmpty(_In_ CDataExchange* pDX, _In_ const CGetFilenameControl& rControl, _In_ UINT nFailureResourceID, _In_ UINT nHelpID = 0xFFFFFFFF);


#endif //#ifndef _DDXFILE_H__
